﻿<%@ Page Language="C#" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Net" %>
<%@ Import Namespace="System.Text" %>
<%@ Import Namespace="System.Collections.Generic" %>

<script runat="server">
    private static readonly string clientId = "<your client id>";
    private static readonly string clientSecret = "<your client secret>";
    private static readonly string redirectUri = "http://localhost:52025/OidcLogin.aspx";
    private static readonly string scope = "openid profile foxconn";
    private static readonly string ssoUrl = "https://sso.foxconn.com/";

    protected void Page_Load(object sender, EventArgs e)
    {
        ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls | (SecurityProtocolType)0xC00 /*Tls12*/;

        string code =  Request.QueryString["code"];
        string state = Request.QueryString["state"];

        if (string.IsNullOrEmpty(code)) {
            if (clientId.StartsWith("<")) {
                Response.Write("請使用你在 https://sso.foxconn.com/Developer/Client 申請成功的ClientId");
                Response.End();
            }
            Response.Redirect(ssoUrl + string.Format(
            "connect/authorize?client_id={0}&redirect_uri={1}&scope={2}",
            clientId,
            redirectUri,
            scope), true);
        }
        string accessToken = GetToken(code);
        // 注意：每一個code只能使用一次。
        Dictionary<string, string> userInfo = GetUserInfo(accessToken);

        // 具體的 Session 操作
        Session["TEST_SSO_OIDC"] = userInfo["name"] + "(" + userInfo["sub"] + "，" + DateTime.Now.ToString("HH:mm:ss") + ")";
        if (string.IsNullOrEmpty(state)) {
            Response.Redirect("~/Content.aspx");
        }
        Response.Redirect(state);
    }


    private static string GetToken(string code)
    {
        string postData = string.Format(
            "client_id={0}&client_secret={1}&grant_type={2}&code={3}&redirect_uri={4}",
            clientId,
            clientSecret,
            "authorization_code",
            code,
            redirectUri);

        byte[] byteArray = Encoding.UTF8.GetBytes(postData);

        HttpWebRequest request = (HttpWebRequest)WebRequest.Create(ssoUrl + "connect/token");
        request.Method = "POST";
        request.ContentType = "application/x-www-form-urlencoded";
        request.ContentLength = byteArray.Length;

        using (Stream dataStream = request.GetRequestStream())
        {
            dataStream.Write(byteArray, 0, byteArray.Length);
        }

        string jsonResponse = "";
        using (WebResponse response = request.GetResponse())
        {
            using (Stream responseStream = response.GetResponseStream())
            {
                using (StreamReader reader = new StreamReader(responseStream))
                {
                    jsonResponse= reader.ReadToEnd();
                }
            }
        }

        // 使用正則表達式解析 JSON
        string pattern = @"(\""|')access_token\1\s*:\s*(\""|')([^'\""]*)\2";
        Regex regex = new Regex(pattern);
        Match match = regex.Match(jsonResponse);
        if (match.Success)
        {
            return match.Groups[3].Value;
        }
        else
        {
            throw new Exception("Access token not found.");
        }
    }


    private Dictionary<string, string>  GetUserInfo(string accessToken)
    {
        HttpWebRequest request = (HttpWebRequest)WebRequest.Create(ssoUrl + "connect/userinfo");
        request.Method = "GET";

        // 設置授權頭，通常使用 Bearer 令牌類型
        request.Headers.Add("Authorization", "Bearer " + accessToken);

        string jsonResponse = "";
        using (WebResponse response = request.GetResponse())
        {
            using (Stream responseStream = response.GetResponseStream())
            {
                using (StreamReader reader = new StreamReader(responseStream))
                {
                    jsonResponse = reader.ReadToEnd();
                }
            }
        }

        // 使用正則表達式解析 JSON
        Dictionary<string, string> userInfo = new Dictionary<string, string>();
        string pattern = @"(\""|')([a-z0-9_]+)\1\s*:\s*(\""|')([^'\""]*)\3";
        Regex regex = new Regex(pattern, RegexOptions.IgnoreCase);
        MatchCollection matches = regex.Matches(jsonResponse);
        foreach (Match match in matches) {
            userInfo[match.Groups[2].Value] = Regex.Unescape(match.Groups[4].Value);
        }
        return userInfo;
    }
</script>